==> bitfront-library-6.3.0.tgz: package/src/app/base/component/store/base-form.component.ts <==
import { HostListener } from "@angular/core";
import { ActivatedRoute } from "@angular/router";
import { Store } from "@ngrx/store";
import { FormGroup } from "@angular/forms";
import { Observable, from } from "rxjs";

import { CustomValidators, MessageService, Item } from "../../../shared";
import { Error } from "../../../shared/data/error";
import { BitToolbarComponent } from "../../../shared/component/bit-toolbar.component";
import { FormType, PageStatus } from "../../../shared/global.constants";

import * as fromModels from "../../models";
import * as fromRootStore from "../../../shared/store";

/**
 * Componente que implementa un formulario para insertar, modificar, eliminar y/o consultar un elemento.
 * T: entidad,
 * V: Definicion de la interface del state
 */
export abstract class BaseFormComponent<T extends Item, V> {
  myForm: FormGroup;
  object: T;
  status$: Observable<PageStatus>;
  toolbar$: Observable<{}>;
  breadcrumb: any[] = []; // contiene la miga de pan de nuestro componente
  formType: FormType;
  soloLectura: boolean; //sera true dependiendo de las circunstacias

  /*tabs properties*/
  myTabs: any = {}; // pestañas definidas en el componente (ver registerTab() e initTabs())
  initialTab: string; // pestaña cargada inicialmente (indicada por parámetro)
  loadedTab: any = {}; // pestañas con datos cargados (todas se cargan de forma perezosa)
  currentTab: number;

  constructor(
    protected store: Store<V>,
    protected activatedRoute: ActivatedRoute,
    protected messageService: MessageService,
    protected factoryCreateAction: fromModels.FactoryCreateAction,
    protected fnCreateNewObject?: fromModels.FactoryCreateNewObject<T>
  ) {
    this.object = this.fnCreateNewObject();
  }

  /** No sabemos a este nivel como construir el FormGroup */
  abstract createFormGroup();

  /** No sabemos a este nivel como iniciarlizar el formulario. Entre otras cosas se creará el breadcrumb */
  abstract initForm(id);

  /** Devuelve true si el formulario edita un elemento existente, o false si inserta un nuevo elemento. */
  get isEditMode(): boolean {
    return this.formType === FormType.Update;
  }
  /**
   * Devuelve true si hay algún valor cambiado en la ficha.
   * Puede ser por un campo del formulario (this.myForm.dirty) o por modificaciones en elementos relacionados.
   */
  get isDataChanged(): boolean {
    return this.myForm && this.myForm.dirty;
  }

  /** Devuelve true si el formulario es válido (campos validados automáticamente, elementos relacionados, etc.). */
  get isFormValid(): boolean {
    return this.myForm && this.myForm.valid;
  }

  /** Evalúa si el usuario puede desactivar/cerrar el componente sin perder cambios todavía no guardados. */
  canDeactivate(): Observable<boolean> | boolean {
    console.log("canDeactivate called");
    if (this.isDataChanged) {
      //si hay cambios, mostramos la ventana de diálogo
      let p = this.messageService.confirm(
        "Hi ha canvis pendents de desar. Està segur de que vol sortir de la plana i perdre-los?"
      );
      let o = from(p);
      return o;
    } else {
      // si no hay cambios, no la mostramos
      console.log("canDeactivate true");
      return true;
    }
  }

  @HostListener("window:beforeunload")
  unloadWithUnsavedChanges($event: any) {
    console.log("unloadWithUnsavedChanges called");
    let msg = "Hi ha canvis pendents de desar. Està segur de que vol sortir de la plana i perdre-los?";
    if (this.isDataChanged) {
      ($event || window.event).returnValue = msg;
      return msg;
    }
    return undefined;
  }

  /** el componente toolbar nos comunica que se ha pulsado un botón */
  onToolbarButtonPressed(button: string) {
    console.log("Button pressed:" + button);
    if (button === BitToolbarComponent.SAVE_BUTTON) {
      this.save();
    } else if (button === BitToolbarComponent.DELETE_BUTTON) {
      this.delete();
    } else if (button === BitToolbarComponent.PRINT_BUTTON) {
      this.handlePrint();
    } else if (button === BitToolbarComponent.HELP_BUTTON) {
      this.help();
    } else {
      this.otherAction(button);
    }
  }

  /** Muestra la ayuda asociada a la pantalla. */
  help(): void {}

  /** el componente toolbar nos comunica que se ha pulsado un botón */
  handlePrint() {
    window.print();
  }

  /** contiene toda la lógica para enviar el formulario al servidor*/

  save(): void {
    console.log("save called");
    if (!this.myForm.valid) {
      let errores = CustomValidators.validateForm(this.myForm);
      this.messageService.sendError(
        new Error(null, "Error al formulari", "Error en el formulari", null, null, true, "", errores)
      );
    } else {
      if (this.preSave()) {
        console.log("presave");
        if (this.isEditMode) {
          console.log("update campanya");
          this.store.dispatch(this.factoryCreateAction(fromModels.GENERIC_TYPE.UPDATE_ITEM, this.object));
        } else {
          console.log("insert campanya");
          this.store.dispatch(this.factoryCreateAction(fromModels.GENERIC_TYPE.INSERT_ITEM, this.object));
        }
      }
    }
  }

  /** función útil que los componentes pueden extender para hacar algo antes de enviar el formulario al servidor */
  preSave(): boolean {
    return true;
  }

  /** contiene la lógica para eliminar el objeto representado en el formulario */
  delete(): void {
    let p = this.messageService.confirm(
      "Està segur que vol eliminar aquest registre? Aquesta acciò no es podrà desfer?"
    );
    let local = this;
    p.then(function(confirmacion) {
      if (confirmacion) {
        local.store.dispatch(local.factoryCreateAction(fromModels.GENERIC_TYPE.DELETE_ITEM, local.object.id));
      }
    });
  }

  /** Cualquier otra accion que no sea la típica de guardar o eliminar */
  otherAction(action: string): void {}

  /** Limpia el formulario. */
  cleanForm(): void {
    this.object = this.fnCreateNewObject(); //mejor User.cleanObject();
  }

  /** Cierra el formulario y vuelve a la pantalla anterior. */
  goBack(): void {
    this.store.dispatch(new fromRootStore.Back());
    //this.router.navigate(["newcampanyas", { query: "true" }]);
  }

  isPreserveQueryParams() {
    return false;
  }

  /** Información de debug. */
  get diagnostic() {
    return JSON.stringify(this.object);
  }

  // Métodos relacionados con pestañas (tabs) //

  /** función para hacer algo si queremos al cambiar de pestaña */
  changeTab(e) {}

  /** Registra una pestaña de visualización de información. */
  registerTab(index: number, id: string) {
    this.myTabs[id] = index;
  }

  // inicialización de pestañas (si hay)
  initTabs() {
    this.currentTab = 0;
    this.initialTab = this.activatedRoute.snapshot.params["tab"];
    for (let tab in this.myTabs) {
      let isInitial = this.isInitialTab(tab);
      this.loadedTab[tab] = isInitial;
      if (isInitial) {
        this.currentTab = this.myTabs[tab];
      }
    }
  }

  /** Indica si la pestaña correspondiente es la inicial. */
  isInitialTab(tab: string) {
    return this.initialTab === tab;
  }

  /** Indica si la pestaña correspondiente está cargada. */
  isLoadedTab(tab: string) {
    return this.loadedTab[tab];
  }
}

==> bitfront-library-6.6.2.tgz: package/src/app/base/component/store/base-form.component.ts <==
import { HostListener } from "@angular/core";
import { ActivatedRoute } from "@angular/router";
import { Store } from "@ngrx/store";
import { FormGroup } from "@angular/forms";
import { Observable, from } from "rxjs";

import { CustomValidators, MessageService, Item } from "../../../shared";
import { Error } from "../../../shared/data/error";
import { BitToolbarComponent } from "../../../shared/component/bit-toolbar.component";
import { FormType, PageStatus } from "../../../shared/global.constants";
import * as fromRootStore from "../../../shared/store";

import * as fromModels from "../../models";

import { BaseComponent } from "./base.component";
/**
 * Componente que implementa un formulario para insertar, modificar, eliminar y/o consultar un elemento.
 * T: entidad,
 * V: Definicion de la interface del state
 */
export abstract class BaseFormComponent<T extends Item, V> extends BaseComponent<V> {
  myForm: FormGroup;
  object: T;
  status$: Observable<PageStatus>;
  toolbar$: Observable<{}>;
  breadcrumb: any[] = []; // contiene la miga de pan de nuestro componente
  formType: FormType;
  soloLectura: boolean; //sera true dependiendo de las circunstacias

  /*tabs properties*/
  myTabs: any = {}; // pestañas definidas en el componente (ver registerTab() e initTabs())
  initialTab: string; // pestaña cargada inicialmente (indicada por parámetro)
  loadedTab: any = {}; // pestañas con datos cargados (todas se cargan de forma perezosa)
  currentTab: number;

  constructor(
    protected store: Store<V>,
    protected activatedRoute: ActivatedRoute,
    protected messageService: MessageService,
    protected factoryCreateAction: fromModels.FactoryCreateAction,
    protected fnCreateNewObject?: fromModels.FactoryCreateNewObject<T>
  ) {
    super(store);
    this.object = this.fnCreateNewObject();
  }

  ngOnDestroy() {
    super.ngOnDestroy();
  }

  /** No sabemos a este nivel como construir el FormGroup */
  abstract createFormGroup();

  /** No sabemos a este nivel como iniciarlizar el formulario. Entre otras cosas se creará el breadcrumb */
  abstract initForm(id);

  /** Devuelve true si el formulario edita un elemento existente, o false si inserta un nuevo elemento. */
  get isEditMode(): boolean {
    return this.formType === FormType.Update;
  }
  /**
   * Devuelve true si hay algún valor cambiado en la ficha.
   * Puede ser por un campo del formulario (this.myForm.dirty) o por modificaciones en elementos relacionados.
   */
  get isDataChanged(): boolean {
    return this.myForm && this.myForm.dirty;
  }

  /** Devuelve true si el formulario es válido (campos validados automáticamente, elementos relacionados, etc.). */
  get isFormValid(): boolean {
    return this.myForm && this.myForm.valid;
  }

  /** Evalúa si el usuario puede desactivar/cerrar el componente sin perder cambios todavía no guardados. */
  canDeactivate(): Observable<boolean> | boolean {
    console.log("canDeactivate called");
    if (this.isDataChanged) {
      //si hay cambios, mostramos la ventana de diálogo
      let p = this.messageService.confirm(
        "Hi ha canvis pendents de desar. Està segur de que vol sortir de la plana i perdre-los?"
      );
      let o = from(p);
      return o;
    } else {
      // si no hay cambios, no la mostramos
      console.log("canDeactivate true");
      return true;
    }
  }

  @HostListener("window:beforeunload")
  unloadWithUnsavedChanges($event: any) {
    console.log("unloadWithUnsavedChanges called");
    let msg = "Hi ha canvis pendents de desar. Està segur de que vol sortir de la plana i perdre-los?";
    if (this.isDataChanged) {
      ($event || window.event).returnValue = msg;
      return msg;
    }
    return undefined;
  }

  /** el componente toolbar nos comunica que se ha pulsado un botón */
  onToolbarButtonPressed(button: string) {
    console.log("Button pressed:" + button);
    if (button === BitToolbarComponent.SAVE_BUTTON) {
      this.save();
    } else if (button === BitToolbarComponent.DELETE_BUTTON) {
      this.delete();
    } else if (button === BitToolbarComponent.PRINT_BUTTON) {
      this.handlePrint();
    } else if (button === BitToolbarComponent.HELP_BUTTON) {
      this.help();
    } else {
      this.otherAction(button);
    }
  }

  /** Muestra la ayuda asociada a la pantalla. */
  help(): void {}

  /** el componente toolbar nos comunica que se ha pulsado un botón */
  handlePrint() {
    window.print();
  }

  /** contiene toda la lógica para enviar el formulario al servidor*/

  save(): void {
    console.log("save called");
    if (!this.myForm.valid) {
      let errores = CustomValidators.validateForm(this.myForm);
      this.messageService.sendError(
        new Error(null, "Error al formulari", "Error en el formulari", null, null, true, "", errores)
      );
    } else {
      if (this.preSave()) {
        console.log("presave");
        if (this.isEditMode) {
          console.log("update campanya");
          this.store.dispatch(this.factoryCreateAction(fromModels.GENERIC_TYPE.UPDATE_ITEM, this.object));
        } else {
          console.log("insert campanya");
          this.store.dispatch(this.factoryCreateAction(fromModels.GENERIC_TYPE.INSERT_ITEM, this.object));
        }
      }
    }
  }

  /** función útil que los componentes pueden extender para hacar algo antes de enviar el formulario al servidor */
  preSave(): boolean {
    return true;
  }

  /** contiene la lógica para eliminar el objeto representado en el formulario */
  delete(): void {
    let p = this.messageService.confirm(
      "Està segur que vol eliminar aquest registre? Aquesta acciò no es podrà desfer?"
    );
    let local = this;
    p.then(function(confirmacion) {
      if (confirmacion) {
        local.store.dispatch(local.factoryCreateAction(fromModels.GENERIC_TYPE.DELETE_ITEM, local.object.id));
      }
    });
  }

  /** Cualquier otra accion que no sea la típica de guardar o eliminar */
  otherAction(action: string): void {}

  /** Limpia el formulario. */
  cleanForm(): void {
    this.object = this.fnCreateNewObject(); //mejor User.cleanObject();
  }

  /** Cierra el formulario y vuelve a la pantalla anterior. */
  goBack(): void {
    this.store.dispatch(new fromRootStore.Back());
    //this.router.navigate(["newcampanyas", { query: "true" }]);
  }

  isPreserveQueryParams() {
    return false;
  }

  /** Información de debug. */
  get diagnostic() {
    return JSON.stringify(this.object);
  }

  // Métodos relacionados con pestañas (tabs) //

  /** función para hacer algo si queremos al cambiar de pestaña */
  changeTab(e) {}

  /** Registra una pestaña de visualización de información. */
  registerTab(index: number, id: string) {
    this.myTabs[id] = index;
  }

  // inicialización de pestañas (si hay)
  initTabs() {
    this.currentTab = 0;
    this.initialTab = this.activatedRoute.snapshot.params["tab"];
    for (let tab in this.myTabs) {
      let isInitial = this.isInitialTab(tab);
      this.loadedTab[tab] = isInitial;
      if (isInitial) {
        this.currentTab = this.myTabs[tab];
      }
    }
  }

  /** Indica si la pestaña correspondiente es la inicial. */
  isInitialTab(tab: string) {
    return this.initialTab === tab;
  }

  /** Indica si la pestaña correspondiente está cargada. */
  isLoadedTab(tab: string) {
    return this.loadedTab[tab];
  }
}
